@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2015 Calculate Ltd. http://www.calculate-linux.org
+# Copyright 2015-2016 Mir Calculate. http://www.calculate-linux.org
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -37,6 +37,7 @@ class Actions(object):
 
     ImageSquash = "squash"
     ImageIso = "iso"
+    ImageContainer = "container"
     ImageMenu = "menu"
     PrepareAssemble = "prepare"
 
@@ -66,6 +67,19 @@ class VariableAcBuilderIso(ActionVariable):
         return "off"
 
 
+class VariableAcBuilderContainer(ActionVariable):
+    """
+    Действие выполняемое при подготовке метаданных контейнера
+    """
+
+    def action(self, cl_action):
+        image_action = self.Get('cl_builder_action')
+        if (cl_action == Actions.Image and
+                image_action in (Actions.ImageContainer,)):
+            return "on"
+        return "off"
+
+
 class VariableAcBuilderSetup(ActionVariable):
     """
     Action variable which has value "on" for setup build directory
@@ -77,7 +91,7 @@ class VariableAcBuilderSetup(ActionVariable):
             return "on"
         image_action = self.Get('cl_builder_action')
         if (cl_action in (Actions.Image,) and
-                    image_action == Actions.PrepareAssemble):
+                image_action == Actions.PrepareAssemble):
             return "on"
         return "off"
 
@@ -92,6 +106,6 @@ class VariableAcBuilderPrepare(ActionVariable):
             return "on"
         image_action = self.Get('cl_builder_action')
         if (cl_action in (Actions.Image,) and
-                    image_action == Actions.PrepareAssemble):
+                image_action == Actions.PrepareAssemble):
             return "on"
         return "off"
